@@ -32,6 +32,8 @@ e_data_book_status_get_type (void)
       { E_DATA_BOOK_STATUS_OTHER_ERROR, "E_DATA_BOOK_STATUS_OTHER_ERROR", "E_DATA_BOOK_STATUS_OTHER_ERROR" },
       { E_DATA_BOOK_STATUS_INVALID_SERVER_VERSION, "E_DATA_BOOK_STATUS_INVALID_SERVER_VERSION", "E_DATA_BOOK_STATUS_INVALID_SERVER_VERSION" },
       { E_DATA_BOOK_STATUS_NO_SPACE, "E_DATA_BOOK_STATUS_NO_SPACE", "E_DATA_BOOK_STATUS_NO_SPACE" },
+      { E_DATA_BOOK_STATUS_INVALID_ARG, "E_DATA_BOOK_STATUS_INVALID_ARG", "E_DATA_BOOK_STATUS_INVALID_ARG" },
+      { E_DATA_BOOK_STATUS_NOT_SUPPORTED, "E_DATA_BOOK_STATUS_NOT_SUPPORTED", "E_DATA_BOOK_STATUS_NOT_SUPPORTED" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("EDataBookStatus", values);
