@@ -43,9 +43,9 @@ contacts_removed (EBookView *book_view, const GList *ids)
 }
 
 static void
-sequence_complete (EBookView *book_view, EBookViewStatus status)
+view_complete (EBookView *book_view, EBookViewStatus status, const gchar *error_msg)
 {
-  printf ("sequence_complete (status == %d)\n", status);
+  printf ("view_complete (status == %d, error_msg == %s%s%s)\n", status, error_msg ? "'" : "", error_msg ? error_msg : "NULL", error_msg ? "'" : "");
 }
 
 gint
@@ -68,23 +68,23 @@ main (gint argc, gchar **argv)
 	book = e_book_new_system_addressbook (NULL);
 	if (!book) {
 		printf ("failed to create ebook\n");
-		exit(0);
+		exit (0);
 	}
 
 	status = e_book_open (book, FALSE, NULL);
 	if (status == FALSE) {
 		printf ("failed to open local addressbook\n");
-		exit(0);
+		exit (0);
 	}
 
 	query = e_book_query_any_field_contains ("");
 
-	for (i = 0; i < 500; i ++) {
+	for (i = 0; i < 500; i++) {
 		status = e_book_get_book_view (book, query, NULL, -1, &new_view, NULL);
 
 		g_signal_connect (new_view, "contacts_added", G_CALLBACK (contacts_added), NULL);
 		g_signal_connect (new_view, "contacts_removed", G_CALLBACK (contacts_removed), NULL);
-		g_signal_connect (new_view, "sequence_complete", G_CALLBACK (sequence_complete), NULL);
+		g_signal_connect (new_view, "view_complete", G_CALLBACK (view_complete), NULL);
 
 		e_book_view_start (new_view);
 
