@@ -85,31 +85,31 @@ struct _CamelMessageContentInfo {
 
 /* system flag bits */
 typedef enum _CamelMessageFlags {
-	CAMEL_MESSAGE_ANSWERED = 1<<0,
-	CAMEL_MESSAGE_DELETED = 1<<1,
-	CAMEL_MESSAGE_DRAFT = 1<<2,
-	CAMEL_MESSAGE_FLAGGED = 1<<3,
-	CAMEL_MESSAGE_SEEN = 1<<4,
+	CAMEL_MESSAGE_ANSWERED = 1 << 0,
+	CAMEL_MESSAGE_DELETED = 1 << 1,
+	CAMEL_MESSAGE_DRAFT = 1 << 2,
+	CAMEL_MESSAGE_FLAGGED = 1 << 3,
+	CAMEL_MESSAGE_SEEN = 1 << 4,
 
 	/* these aren't really system flag bits, but are convenience flags */
-	CAMEL_MESSAGE_ATTACHMENTS = 1<<5,
-	CAMEL_MESSAGE_ANSWERED_ALL = 1<<6,
-	CAMEL_MESSAGE_JUNK = 1<<7,
-	CAMEL_MESSAGE_SECURE = 1<<8,
-	CAMEL_MESSAGE_USER_NOT_DELETABLE = 1<<9,
-	CAMEL_MESSAGE_HIDDEN = 1<<10,
-	CAMEL_MESSAGE_NOTJUNK = 1<<11,
-	CAMEL_MESSAGE_FORWARDED = 1<<12,
+	CAMEL_MESSAGE_ATTACHMENTS = 1 << 5,
+	CAMEL_MESSAGE_ANSWERED_ALL = 1 << 6,
+	CAMEL_MESSAGE_JUNK = 1 << 7,
+	CAMEL_MESSAGE_SECURE = 1 << 8,
+	CAMEL_MESSAGE_USER_NOT_DELETABLE = 1 << 9,
+	CAMEL_MESSAGE_HIDDEN = 1 << 10,
+	CAMEL_MESSAGE_NOTJUNK = 1 << 11,
+	CAMEL_MESSAGE_FORWARDED = 1 << 12,
 
 	/* following flags are for the folder, and are not really permanent flags */
-	CAMEL_MESSAGE_FOLDER_FLAGGED = 1<<16, /* for use by the folder implementation */
-	/* flags after 1<<16 are used by camel providers,
+	CAMEL_MESSAGE_FOLDER_FLAGGED = 1 << 16, /* for use by the folder implementation */
+	/* flags after 1 << 16 are used by camel providers,
            if adding non permanent flags, add them to the end  */
 
-	CAMEL_MESSAGE_JUNK_LEARN = 1<<30, /* used when setting CAMEL_MESSAGE_JUNK flag
+	CAMEL_MESSAGE_JUNK_LEARN = 1 << 30, /* used when setting CAMEL_MESSAGE_JUNK flag
 					     to say that we request junk plugin
 					     to learn that message as junk/non junk */
-	CAMEL_MESSAGE_USER = 1<<31 /* supports user flags */
+	CAMEL_MESSAGE_USER = 1 << 31 /* supports user flags */
 } CamelMessageFlags;
 
 /* Changes to system flags will NOT trigger a folder changed event */
@@ -196,7 +196,7 @@ struct _CamelMessageInfoBase {
 	const gchar *cc;
 	const gchar *mlist;
 
-	guint32 flags;
+	CamelMessageFlags flags;
 	guint32 size;
 
 	time_t date_sent;
@@ -215,10 +215,15 @@ struct _CamelMessageInfoBase {
 	gchar *bodystructure;
 };
 
-typedef enum _CamelFolderSummaryFlags {
-	CAMEL_SUMMARY_DIRTY = 1<<0
+typedef enum {
+	CAMEL_SUMMARY_DIRTY = 1 << 0
 } CamelFolderSummaryFlags;
 
+/**
+ * CamelFolderSummaryLock:
+ *
+ * Since: 2.32
+ **/
 typedef enum _CamelFolderSummaryLock {
 	CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK,
 	CAMEL_FOLDER_SUMMARY_IO_LOCK,
@@ -233,7 +238,7 @@ struct _CamelFolderSummary {
 
 	/* header info */
 	guint32 version;	/* version of file loaded/loading */
-	guint32 flags;		/* flags */
+	CamelFolderSummaryFlags flags;
 	guint32 nextuid;	/* next uid? */
 	time_t time;		/* timestamp for this summary (for implementors to use) */
 	guint32 saved_count;	/* how many were saved/loaded */
@@ -333,10 +338,10 @@ GType			 camel_folder_summary_get_type	(void);
 CamelFolderSummary      *camel_folder_summary_new	(struct _CamelFolder *folder);
 
 /* Deprecated */
-void camel_folder_summary_set_filename(CamelFolderSummary *summary, const gchar *filename);
+void camel_folder_summary_set_filename (CamelFolderSummary *summary, const gchar *filename);
 
-void camel_folder_summary_set_index(CamelFolderSummary *summary, CamelIndex *index);
-void camel_folder_summary_set_build_content(CamelFolderSummary *summary, gboolean state);
+void camel_folder_summary_set_index (CamelFolderSummary *summary, CamelIndex *index);
+void camel_folder_summary_set_build_content (CamelFolderSummary *summary, gboolean state);
 
 guint32  camel_folder_summary_next_uid        (CamelFolderSummary *summary);
 gchar    *camel_folder_summary_next_uid_string (CamelFolderSummary *summary);
@@ -347,12 +352,12 @@ gint camel_folder_summary_save_to_db (CamelFolderSummary *s, GError **error);
 gint camel_folder_summary_load_from_db (CamelFolderSummary *s, GError **error);
 
 /* only load the header */
-gint camel_folder_summary_header_load(CamelFolderSummary *summary);
+gint camel_folder_summary_header_load (CamelFolderSummary *summary);
 gint camel_folder_summary_header_load_from_db (CamelFolderSummary *s, struct _CamelStore *store, const gchar *folder_name, GError **error);
 gint camel_folder_summary_header_save_to_db (CamelFolderSummary *s, GError **error);
 
 /* set the dirty bit on the summary */
-void camel_folder_summary_touch(CamelFolderSummary *summary);
+void camel_folder_summary_touch (CamelFolderSummary *summary);
 
 /* add a new raw summary item */
 void camel_folder_summary_add (CamelFolderSummary *summary, CamelMessageInfo *info);
@@ -371,74 +376,76 @@ void camel_folder_summary_remove_index_fast (CamelFolderSummary *s, gint index);
 void camel_folder_summary_remove_uid_fast (CamelFolderSummary *s, const gchar *uid);
 
 /* build/add raw summary items */
-CamelMessageInfo *camel_folder_summary_add_from_header(CamelFolderSummary *summary, struct _camel_header_raw *headers);
-CamelMessageInfo *camel_folder_summary_add_from_parser(CamelFolderSummary *summary, CamelMimeParser *parser);
-CamelMessageInfo *camel_folder_summary_add_from_message(CamelFolderSummary *summary, CamelMimeMessage *message);
+CamelMessageInfo *camel_folder_summary_add_from_header (CamelFolderSummary *summary, struct _camel_header_raw *headers);
+CamelMessageInfo *camel_folder_summary_add_from_parser (CamelFolderSummary *summary, CamelMimeParser *parser);
+CamelMessageInfo *camel_folder_summary_add_from_message (CamelFolderSummary *summary, CamelMimeMessage *message);
 
 /* Just build raw summary items */
-CamelMessageInfo *camel_folder_summary_info_new_from_header(CamelFolderSummary *summary, struct _camel_header_raw *headers);
-CamelMessageInfo *camel_folder_summary_info_new_from_parser(CamelFolderSummary *summary, CamelMimeParser *parser);
-CamelMessageInfo *camel_folder_summary_info_new_from_message(CamelFolderSummary *summary, CamelMimeMessage *message, const gchar *bodystructure);
+CamelMessageInfo *camel_folder_summary_info_new_from_header (CamelFolderSummary *summary, struct _camel_header_raw *headers);
+CamelMessageInfo *camel_folder_summary_info_new_from_parser (CamelFolderSummary *summary, CamelMimeParser *parser);
+CamelMessageInfo *camel_folder_summary_info_new_from_message (CamelFolderSummary *summary, CamelMimeMessage *message, const gchar *bodystructure);
 
-CamelMessageContentInfo *camel_folder_summary_content_info_new(CamelFolderSummary *summary);
-void camel_folder_summary_content_info_free(CamelFolderSummary *summary, CamelMessageContentInfo *ci);
+CamelMessageContentInfo *camel_folder_summary_content_info_new (CamelFolderSummary *summary);
+void camel_folder_summary_content_info_free (CamelFolderSummary *summary, CamelMessageContentInfo *ci);
 
 /* removes a summary item, doesn't fix content offsets */
-void camel_folder_summary_remove(CamelFolderSummary *summary, CamelMessageInfo *info);
-void camel_folder_summary_remove_uid(CamelFolderSummary *summary, const gchar *uid);
-void camel_folder_summary_remove_index(CamelFolderSummary *summary, gint index);
-void camel_folder_summary_remove_range(CamelFolderSummary *summary, gint start, gint end);
+void camel_folder_summary_remove (CamelFolderSummary *summary, CamelMessageInfo *info);
+void camel_folder_summary_remove_uid (CamelFolderSummary *summary, const gchar *uid);
+void camel_folder_summary_remove_index (CamelFolderSummary *summary, gint index);
+void camel_folder_summary_remove_range (CamelFolderSummary *summary, gint start, gint end);
 
 /* remove all items */
-void camel_folder_summary_clear(CamelFolderSummary *summary);
+void camel_folder_summary_clear (CamelFolderSummary *summary);
 void camel_folder_summary_clear_db (CamelFolderSummary *s);
 
+/* update visible/unread/... counts based on message flags */
+void camel_folder_summary_update_counts_by_flags (CamelFolderSummary *summary, guint32 flags, gboolean subtract);
+
 /* lookup functions */
-guint camel_folder_summary_count(CamelFolderSummary *summary);
-CamelMessageInfo *camel_folder_summary_index(CamelFolderSummary *summary, gint index);
-CamelMessageInfo *camel_folder_summary_uid(CamelFolderSummary *summary, const gchar *uid);
+guint camel_folder_summary_count (CamelFolderSummary *summary);
+CamelMessageInfo *camel_folder_summary_index (CamelFolderSummary *summary, gint index);
+CamelMessageInfo *camel_folder_summary_uid (CamelFolderSummary *summary, const gchar *uid);
 gchar * camel_folder_summary_uid_from_index (CamelFolderSummary *s, gint i);
 gboolean camel_folder_summary_check_uid (CamelFolderSummary *s, const gchar *uid);
 
-GPtrArray *camel_folder_summary_array(CamelFolderSummary *summary);
-GHashTable *camel_folder_summary_get_hashtable(CamelFolderSummary *s);
+GPtrArray *camel_folder_summary_array (CamelFolderSummary *summary);
+GHashTable *camel_folder_summary_get_hashtable (CamelFolderSummary *s);
 void camel_folder_summary_free_hashtable (GHashTable *ht);
-GHashTable *camel_folder_summary_get_flag_cache (CamelFolderSummary *summary);
-void camel_folder_summary_update_flag_cache (CamelFolderSummary *s, const gchar *uid, guint32 flag);
 
 /* basically like strings, but certain keywords can be compressed and de-cased */
-gint camel_folder_summary_encode_token(FILE *out, const gchar *str);
-gint camel_folder_summary_decode_token(FILE *in, gchar **str);
+gint camel_folder_summary_encode_token (FILE *out, const gchar *str);
+gint camel_folder_summary_decode_token (FILE *in, gchar **str);
 
 /* message flag operations */
-gboolean	camel_flag_get(CamelFlag **list, const gchar *name);
-gboolean	camel_flag_set(CamelFlag **list, const gchar *name, gboolean value);
-gboolean	camel_flag_list_copy(CamelFlag **to, CamelFlag **from);
-gint		camel_flag_list_size(CamelFlag **list);
-void		camel_flag_list_free(CamelFlag **list);
+gboolean	camel_flag_get (CamelFlag **list, const gchar *name);
+gboolean	camel_flag_set (CamelFlag **list, const gchar *name, gboolean value);
+gboolean	camel_flag_list_copy (CamelFlag **to, CamelFlag **from);
+gint		camel_flag_list_size (CamelFlag **list);
+void		camel_flag_list_free (CamelFlag **list);
 
-guint32         camel_system_flag (const gchar *name);
-gboolean        camel_system_flag_get (guint32 flags, const gchar *name);
+CamelMessageFlags
+		camel_system_flag (const gchar *name);
+gboolean	camel_system_flag_get (CamelMessageFlags flags, const gchar *name);
 
 /* message tag operations */
-const gchar	*camel_tag_get(CamelTag **list, const gchar *name);
-gboolean	camel_tag_set(CamelTag **list, const gchar *name, const gchar *value);
-gboolean	camel_tag_list_copy(CamelTag **to, CamelTag **from);
-gint		camel_tag_list_size(CamelTag **list);
-void		camel_tag_list_free(CamelTag **list);
+const gchar	*camel_tag_get (CamelTag **list, const gchar *name);
+gboolean	camel_tag_set (CamelTag **list, const gchar *name, const gchar *value);
+gboolean	camel_tag_list_copy (CamelTag **to, CamelTag **from);
+gint		camel_tag_list_size (CamelTag **list);
+void		camel_tag_list_free (CamelTag **list);
 
 /* Summary may be null */
 /* Use anonymous pointers to avoid tons of cast crap */
-gpointer camel_message_info_new(CamelFolderSummary *summary);
-void camel_message_info_ref(gpointer info);
-CamelMessageInfo *camel_message_info_new_from_header(CamelFolderSummary *summary, struct _camel_header_raw *header);
-void camel_message_info_free(gpointer info);
-gpointer camel_message_info_clone(gconstpointer info);
+gpointer camel_message_info_new (CamelFolderSummary *summary);
+gpointer camel_message_info_ref (gpointer info);
+CamelMessageInfo *camel_message_info_new_from_header (CamelFolderSummary *summary, struct _camel_header_raw *header);
+void camel_message_info_free (gpointer info);
+gpointer camel_message_info_clone (gconstpointer info);
 
 /* accessors */
-gconstpointer camel_message_info_ptr(const CamelMessageInfo *mi, gint id);
-guint32 camel_message_info_uint32(const CamelMessageInfo *mi, gint id);
-time_t camel_message_info_time(const CamelMessageInfo *mi, gint id);
+gconstpointer camel_message_info_ptr (const CamelMessageInfo *mi, gint id);
+guint32 camel_message_info_uint32 (const CamelMessageInfo *mi, gint id);
+time_t camel_message_info_time (const CamelMessageInfo *mi, gint id);
 
 #define camel_message_info_uid(mi) ((const gchar *)((const CamelMessageInfo *)mi)->uid)
 
@@ -490,12 +497,12 @@ time_t camel_message_info_time(const CamelMessageInfo *mi, gint id);
  **/
 #define camel_message_info_content(mi) ((const CamelMessageContentInfo *)camel_message_info_ptr((const CamelMessageInfo *)mi, CAMEL_MESSAGE_INFO_CONTENT))
 
-gboolean camel_message_info_user_flag(const CamelMessageInfo *mi, const gchar *id);
-const gchar *camel_message_info_user_tag(const CamelMessageInfo *mi, const gchar *id);
+gboolean camel_message_info_user_flag (const CamelMessageInfo *mi, const gchar *id);
+const gchar *camel_message_info_user_tag (const CamelMessageInfo *mi, const gchar *id);
 
-gboolean camel_message_info_set_flags(CamelMessageInfo *mi, guint32 flags, guint32 set);
-gboolean camel_message_info_set_user_flag(CamelMessageInfo *mi, const gchar *id, gboolean state);
-gboolean camel_message_info_set_user_tag(CamelMessageInfo *mi, const gchar *id, const gchar *val);
+gboolean camel_message_info_set_flags (CamelMessageInfo *mi, CamelMessageFlags flags, guint32 set);
+gboolean camel_message_info_set_user_flag (CamelMessageInfo *mi, const gchar *id, gboolean state);
+gboolean camel_message_info_set_user_tag (CamelMessageInfo *mi, const gchar *id, const gchar *val);
 
 void camel_folder_summary_set_need_preview (CamelFolderSummary *summary, gboolean preview);
 void camel_folder_summary_add_preview (CamelFolderSummary *s, CamelMessageInfo *info);
@@ -509,7 +516,7 @@ void camel_content_info_dump (CamelMessageContentInfo *ci, gint depth);
 void camel_message_info_dump (CamelMessageInfo *mi);
 
 /* Migration code */
-gint camel_folder_summary_migrate_infos(CamelFolderSummary *s);
+gint camel_folder_summary_migrate_infos (CamelFolderSummary *s);
 
 void camel_folder_summary_lock   (CamelFolderSummary *summary, CamelFolderSummaryLock lock);
 void camel_folder_summary_unlock (CamelFolderSummary *summary, CamelFolderSummaryLock lock);
