@@ -54,13 +54,17 @@
 	       #include <stdio.h>*/
 
 /* simple data wrapper */
-static void
-simple_data_wrapper_construct_from_parser (CamelDataWrapper *dw, CamelMimeParser *mp)
+static gboolean
+simple_data_wrapper_construct_from_parser (CamelDataWrapper *dw,
+                                           CamelMimeParser *mp,
+                                           GCancellable *cancellable,
+                                           GError **error)
 {
 	gchar *buf;
 	GByteArray *buffer;
 	CamelStream *mem;
 	gsize len;
+	gboolean success;
 
 	d(printf ("simple_data_wrapper_construct_from_parser()\n"));
 
@@ -74,8 +78,11 @@ simple_data_wrapper_construct_from_parser (CamelDataWrapper *dw, CamelMimeParser
 	d(printf("message part kept in memory!\n"));
 
 	mem = camel_stream_mem_new_with_byte_array (buffer);
-	camel_data_wrapper_construct_from_stream (dw, mem);
+	success = camel_data_wrapper_construct_from_stream_sync (
+		dw, mem, cancellable, error);
 	g_object_unref (mem);
+
+	return success;
 }
 
 /**
@@ -83,14 +90,18 @@ simple_data_wrapper_construct_from_parser (CamelDataWrapper *dw, CamelMimeParser
  *
  * Since: 2.24
  **/
-void
-camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParser *mp)
+gboolean
+camel_mime_part_construct_content_from_parser (CamelMimePart *dw,
+                                               CamelMimeParser *mp,
+                                               GCancellable *cancellable,
+                                               GError **error)
 {
 	CamelDataWrapper *content = NULL;
 	CamelContentType *ct;
 	gchar *encoding;
+	gboolean success = TRUE;
 
-	g_return_if_fail (CAMEL_IS_MIME_PART (dw));
+	g_return_val_if_fail (CAMEL_IS_MIME_PART (dw), FALSE);
 
 	ct = camel_mime_parser_content_type (mp);
 
@@ -105,13 +116,15 @@ camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParse
 			camel_multipart_construct_from_parser ((CamelMultipart *) content, mp);
 		} else {
 			content = camel_data_wrapper_new ();
-			simple_data_wrapper_construct_from_parser (content, mp);
+			success = simple_data_wrapper_construct_from_parser (
+				content, mp, cancellable, error);
 		}
 		break;
 	case CAMEL_MIME_PARSER_STATE_MESSAGE:
 		d(printf("Creating message part\n"));
 		content = (CamelDataWrapper *) camel_mime_message_new ();
-		camel_mime_part_construct_from_parser ((CamelMimePart *)content, mp);
+		success = camel_mime_part_construct_from_parser_sync (
+			(CamelMimePart *)content, mp, cancellable, error);
 		break;
 	case CAMEL_MIME_PARSER_STATE_MULTIPART:
 		d(printf("Creating multi-part\n"));
@@ -122,7 +135,7 @@ camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParse
 		else
 			content = (CamelDataWrapper *) camel_multipart_new ();
 
-		camel_multipart_construct_from_parser((CamelMultipart *)content, mp);
+		camel_multipart_construct_from_parser ((CamelMultipart *)content, mp);
 		d(printf("Created multi-part\n"));
 		break;
 	default:
@@ -140,11 +153,19 @@ camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParse
 	}
 
 	g_free (encoding);
+
+	return success;
 }
 
 /**
  * camel_mime_message_build_preview:
  *
+ * <note>
+ *   <para>
+ *     This function blocks like crazy.
+ *   </para>
+ * </note>
+ *
  * Since: 2.28
  **/
 gboolean
@@ -159,11 +180,11 @@ camel_mime_message_build_preview (CamelMimePart *msg,
 		gint i, nparts;
 		CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content ((CamelMedium *)msg);
 
-		if (!CAMEL_IS_MULTIPART(mp))
+		if (!CAMEL_IS_MULTIPART (mp))
 			g_assert (0);
-		nparts = camel_multipart_get_number(mp);
+		nparts = camel_multipart_get_number (mp);
 		for (i = 0; i < nparts && !got_plain; i++) {
-			CamelMimePart *part = camel_multipart_get_part(mp, i);
+			CamelMimePart *part = camel_multipart_get_part (mp, i);
 			got_plain = camel_mime_message_build_preview (part, info);
 		}
 
@@ -171,47 +192,49 @@ camel_mime_message_build_preview (CamelMimePart *msg,
 		/*    !camel_content_type_is (dw->mime_type, "text", "html") && */
 		    !camel_content_type_is (dw->mime_type, "text", "calendar")) {
 		CamelStream *mstream, *bstream;
-		mstream = camel_stream_mem_new();
-		if (camel_data_wrapper_decode_to_stream (dw, mstream) > 0) {
+
+		/* FIXME Pass a GCancellable and GError here. */
+		mstream = camel_stream_mem_new ();
+		if (camel_data_wrapper_decode_to_stream_sync (dw, mstream, NULL, NULL) > 0) {
 			gchar *line = NULL;
 			GString *str = g_string_new (NULL);
 
-			camel_stream_reset (mstream);
+			camel_stream_reset (mstream, NULL);
 			bstream = camel_stream_buffer_new (mstream, CAMEL_STREAM_BUFFER_READ|CAMEL_STREAM_BUFFER_BUFFER);
 
 			/* We should fetch just 200 unquoted lines. */
-			while ((line = camel_stream_buffer_read_line((CamelStreamBuffer *)bstream)) && str->len < 200) {
+			while ((line = camel_stream_buffer_read_line ((CamelStreamBuffer *)bstream, NULL, NULL)) && str->len < 200) {
 				gchar *tmp = line;
 				if (!line)
 					continue;
 
 				if (*line == '>' || strstr(line, "wrote:")) {
-					g_free(tmp);
+					g_free (tmp);
 					continue;
 				}
 				if (g_str_has_prefix (line, "--")) {
-					g_free(tmp);
+					g_free (tmp);
 					line = NULL;
 					break;
 				}
 				while (*line && ((*line == ' ') || *line == '\t'))
 					line++;
 				if (*line == '\0' || *line == '\n') {
-					g_free(tmp);
+					g_free (tmp);
 					continue;
 				}
 
 				g_string_append (str, " ");
 				g_string_append (str, line);
-				g_free(tmp);
+				g_free (tmp);
 				line = NULL;
 			}
 			if (str->len > 100) {
 				g_string_insert (str, 100, "\n");
 			}
 			/* We don't mark dirty, as we don't store these */
-			((CamelMessageInfoBase *) info)->preview = camel_utf8_make_valid(str->str);
-			g_string_free(str, TRUE);
+			((CamelMessageInfoBase *) info)->preview = camel_utf8_make_valid (str->str);
+			g_string_free (str, TRUE);
 
 			g_object_unref (bstream);
 		}
