@@ -92,6 +92,7 @@ static gssize
 stream_filter_read (CamelStream *stream,
                     gchar *buffer,
                     gsize n,
+                    GCancellable *cancellable,
                     GError **error)
 {
 	CamelStreamFilterPrivate *priv;
@@ -102,16 +103,17 @@ stream_filter_read (CamelStream *stream,
 
 	priv->last_was_read = TRUE;
 
-	g_check(priv->realbuffer);
+	g_check (priv->realbuffer);
 
 	if (priv->filteredlen<=0) {
 		gsize presize = READ_PAD;
 
 		size = camel_stream_read (
-			priv->source, priv->buffer, READ_SIZE, error);
+			priv->source, priv->buffer,
+			READ_SIZE, cancellable, error);
 		if (size <= 0) {
 			/* this is somewhat untested */
-			if (camel_stream_eos(priv->source)) {
+			if (camel_stream_eos (priv->source)) {
 				f = priv->filters;
 				priv->filtered = priv->buffer;
 				priv->filteredlen = 0;
@@ -119,7 +121,7 @@ stream_filter_read (CamelStream *stream,
 					camel_mime_filter_complete (
 						f->filter, priv->filtered, priv->filteredlen,
 						presize, &priv->filtered, &priv->filteredlen, &presize);
-					g_check(priv->realbuffer);
+					g_check (priv->realbuffer);
 					f = f->next;
 				}
 				size = priv->filteredlen;
@@ -133,17 +135,17 @@ stream_filter_read (CamelStream *stream,
 			priv->filteredlen = size;
 
 			d(printf ("\n\nOriginal content (%s): '", ((CamelObject *)priv->source)->class->name));
-			d(fwrite(priv->filtered, sizeof(gchar), priv->filteredlen, stdout));
+			d (fwrite (priv->filtered, sizeof (gchar), priv->filteredlen, stdout));
 			d(printf("'\n"));
 
 			while (f) {
 				camel_mime_filter_filter (
 					f->filter, priv->filtered, priv->filteredlen, presize,
 					&priv->filtered, &priv->filteredlen, &presize);
-				g_check(priv->realbuffer);
+				g_check (priv->realbuffer);
 
 				d(printf ("Filtered content (%s): '", ((CamelObject *)f->filter)->class->name));
-				d(fwrite(priv->filtered, sizeof(gchar), priv->filteredlen, stdout));
+				d (fwrite (priv->filtered, sizeof (gchar), priv->filteredlen, stdout));
 				d(printf("'\n"));
 
 				f = f->next;
@@ -151,12 +153,12 @@ stream_filter_read (CamelStream *stream,
 		}
 	}
 
-	size = MIN(n, priv->filteredlen);
-	memcpy(buffer, priv->filtered, size);
+	size = MIN (n, priv->filteredlen);
+	memcpy (buffer, priv->filtered, size);
 	priv->filteredlen -= size;
 	priv->filtered += size;
 
-	g_check(priv->realbuffer);
+	g_check (priv->realbuffer);
 
 	return size;
 }
@@ -168,6 +170,7 @@ static gssize
 stream_filter_write (CamelStream *stream,
                      const gchar *buf,
                      gsize n,
+                     GCancellable *cancellable,
                      GError **error)
 {
 	CamelStreamFilterPrivate *priv;
@@ -180,44 +183,45 @@ stream_filter_write (CamelStream *stream,
 	priv->last_was_read = FALSE;
 
 	d(printf ("\n\nWriting: Original content (%s): '", ((CamelObject *)priv->source)->class->name));
-	d(fwrite(buf, sizeof(gchar), n, stdout));
+	d (fwrite (buf, sizeof (gchar), n, stdout));
 	d(printf("'\n"));
 
-	g_check(priv->realbuffer);
+	g_check (priv->realbuffer);
 
 	while (left) {
 		/* Sigh, since filters expect non const args, copy the input first, we do this in handy sized chunks */
-		len = MIN(READ_SIZE, left);
+		len = MIN (READ_SIZE, left);
 		buffer = realbuffer + READ_PAD;
-		memcpy(buffer, buf, len);
+		memcpy (buffer, buf, len);
 		buf += len;
 		left -= len;
 
 		f = priv->filters;
 		presize = READ_PAD;
 		while (f) {
-			camel_mime_filter_filter(f->filter, buffer, len, presize, &buffer, &len, &presize);
+			camel_mime_filter_filter (f->filter, buffer, len, presize, &buffer, &len, &presize);
 
-			g_check(priv->realbuffer);
+			g_check (priv->realbuffer);
 
 			d(printf ("Filtered content (%s): '", ((CamelObject *)f->filter)->class->name));
-			d(fwrite(buffer, sizeof(gchar), len, stdout));
+			d (fwrite (buffer, sizeof (gchar), len, stdout));
 			d(printf("'\n"));
 
 			f = f->next;
 		}
 
-		if (camel_stream_write (priv->source, buffer, len, error) != len)
+		if (camel_stream_write (priv->source, buffer, len, cancellable, error) != len)
 			return -1;
 	}
 
-	g_check(priv->realbuffer);
+	g_check (priv->realbuffer);
 
 	return n;
 }
 
 static gint
 stream_filter_flush (CamelStream *stream,
+                     GCancellable *cancellable,
                      GError **error)
 {
 	CamelStreamFilterPrivate *priv;
@@ -237,39 +241,39 @@ stream_filter_flush (CamelStream *stream,
 	f = priv->filters;
 
 	d(printf ("\n\nFlushing: Original content (%s): '", ((CamelObject *)priv->source)->class->name));
-	d(fwrite(buffer, sizeof(gchar), len, stdout));
+	d (fwrite (buffer, sizeof (gchar), len, stdout));
 	d(printf("'\n"));
 
 	while (f) {
-		camel_mime_filter_complete(f->filter, buffer, len, presize, &buffer, &len, &presize);
+		camel_mime_filter_complete (f->filter, buffer, len, presize, &buffer, &len, &presize);
 
 		d(printf ("Filtered content (%s): '", ((CamelObject *)f->filter)->class->name));
-		d(fwrite(buffer, sizeof(gchar), len, stdout));
+		d (fwrite (buffer, sizeof (gchar), len, stdout));
 		d(printf("'\n"));
 
 		f = f->next;
 	}
 
-	if (len > 0 && camel_stream_write (priv->source, buffer, len, error) == -1)
+	if (len > 0 && camel_stream_write (priv->source, buffer, len, cancellable, error) == -1)
 		return -1;
 
-	return camel_stream_flush (priv->source, error);
+	return camel_stream_flush (priv->source, cancellable, error);
 }
 
 static gint
 stream_filter_close (CamelStream *stream,
+                     GCancellable *cancellable,
                      GError **error)
 {
 	CamelStreamFilterPrivate *priv;
 
 	priv = CAMEL_STREAM_FILTER_GET_PRIVATE (stream);
 
-	if (!priv->last_was_read) {
-		if (stream_filter_flush (stream, error) == -1)
-			return -1;
-	}
+	/* Ignore errors while flushing. */
+	if (!priv->last_was_read)
+		stream_filter_flush (stream, cancellable, NULL);
 
-	return camel_stream_close (priv->source, error);
+	return camel_stream_close (priv->source, cancellable, error);
 }
 
 static gboolean
@@ -335,7 +339,7 @@ camel_stream_filter_init (CamelStreamFilter *stream)
 {
 	stream->priv = CAMEL_STREAM_FILTER_GET_PRIVATE (stream);
 
-	stream->priv->realbuffer = g_malloc(READ_SIZE + READ_PAD);
+	stream->priv->realbuffer = g_malloc (READ_SIZE + READ_PAD);
 	stream->priv->buffer = stream->priv->realbuffer + READ_PAD;
 	stream->priv->last_was_read = TRUE;
 	stream->priv->flushed = FALSE;
@@ -348,7 +352,7 @@ camel_stream_filter_init (CamelStreamFilter *stream)
  *
  * Returns: a new #CamelStreamFilter object.
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 CamelStream *
 camel_stream_filter_new (CamelStream *source)
@@ -370,7 +374,7 @@ camel_stream_filter_new (CamelStream *source)
  * camel_stream_filter_get_source:
  * @stream: a #CamelStreamFilter
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 CamelStream *
 camel_stream_filter_get_source (CamelStreamFilter *stream)
@@ -405,7 +409,7 @@ camel_stream_filter_add (CamelStreamFilter *stream,
 
 	priv = CAMEL_STREAM_FILTER_GET_PRIVATE (stream);
 
-	fn = g_malloc(sizeof(*fn));
+	fn = g_malloc (sizeof (*fn));
 	fn->id = priv->filterid++;
 	fn->filter = g_object_ref (filter);
 
@@ -442,7 +446,7 @@ camel_stream_filter_remove (CamelStreamFilter *stream,
 		if (fn->id == id) {
 			f->next = fn->next;
 			g_object_unref (fn->filter);
-			g_free(fn);
+			g_free (fn);
 		}
 		f = f->next;
 	}
