@@ -20,16 +20,29 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef _CAMEL_IMAP_SUMMARY_H
-#define _CAMEL_IMAP_SUMMARY_H
+#ifndef CAMEL_IMAP_SUMMARY_H
+#define CAMEL_IMAP_SUMMARY_H
 
-#include "camel-imap-types.h"
-#include <camel/camel-folder-summary.h>
-#include <camel/camel-exception.h>
+#include <camel/camel.h>
 
-#define CAMEL_IMAP_SUMMARY(obj)         CAMEL_CHECK_CAST (obj, camel_imap_summary_get_type (), CamelImapSummary)
-#define CAMEL_IMAP_SUMMARY_CLASS(klass) CAMEL_CHECK_CLASS_CAST (klass, camel_imap_summary_get_type (), CamelImapSummaryClass)
-#define CAMEL_IS_IMAP_SUMMARY(obj)      CAMEL_CHECK_TYPE (obj, camel_imap_summary_get_type ())
+/* Standard GObject macros */
+#define CAMEL_TYPE_IMAP_SUMMARY \
+	(camel_imap_summary_get_type ())
+#define CAMEL_IMAP_SUMMARY(obj) \
+	(G_TYPE_CHECK_INSTANCE_CAST \
+	((obj), CAMEL_TYPE_IMAP_SUMMARY, CamelImapSummary))
+#define CAMEL_IMAP_SUMMARY_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_CAST \
+	((cls), CAMEL_TYPE_IMAP_SUMMARY, CamelImapSummaryClass))
+#define CAMEL_IS_IMAP_SUMMARY(obj) \
+	(G_TYPE_CHECK_INSTANCE_TYPE \
+	((obj), CAMEL_TYPE_IMAP_SUMMARY))
+#define CAMEL_IS_IMAP_SUMMARY_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_TYPE \
+	((cls), CAMEL_TYPE_IMAP_SUMMARY))
+#define CAMEL_IMAP_SUMMARY_GET_CLASS(obj) \
+	(G_TYPE_INSTANCE_GET_CLASS \
+	((obj), CAMEL_TYPE_IMAP_SUMMARY, CamelImapSummaryClass))
 
 #define CAMEL_IMAP_SERVER_FLAGS (CAMEL_MESSAGE_ANSWERED | \
 				 CAMEL_MESSAGE_DELETED | \
@@ -40,9 +53,10 @@
 G_BEGIN_DECLS
 
 enum {
-	CAMEL_IMAP_MESSAGE_RECENT = 1<<17
+	CAMEL_IMAP_MESSAGE_RECENT = 1 << 17
 };
 
+typedef struct _CamelImapSummary CamelImapSummary;
 typedef struct _CamelImapSummaryClass CamelImapSummaryClass;
 
 typedef struct _CamelImapMessageContentInfo {
@@ -68,18 +82,19 @@ struct _CamelImapSummaryClass {
 
 };
 
-CamelType               camel_imap_summary_get_type     (void);
-CamelFolderSummary *camel_imap_summary_new          (struct _CamelFolder *folder, const gchar *filename);
-
-void camel_imap_summary_add_offline (CamelFolderSummary *summary,
-				     const gchar *uid,
-				     CamelMimeMessage *message,
-				     const CamelMessageInfo *info);
-
-void camel_imap_summary_add_offline_uncached (CamelFolderSummary *summary,
-					      const gchar *uid,
-					      const CamelMessageInfo *info);
+GType		camel_imap_summary_get_type	(void);
+CamelFolderSummary *
+		camel_imap_summary_new		(CamelFolder *folder,
+						 const gchar *filename);
+void		camel_imap_summary_add_offline	(CamelFolderSummary *summary,
+						 const gchar *uid,
+						 CamelMimeMessage *message,
+						 const CamelMessageInfo *info);
+void		camel_imap_summary_add_offline_uncached
+						(CamelFolderSummary *summary,
+						 const gchar *uid,
+						 const CamelMessageInfo *info);
 
 G_END_DECLS
 
-#endif /* _CAMEL_IMAP_SUMMARY_H */
+#endif /* CAMEL_IMAP_SUMMARY_H */
