@@ -9,11 +9,6 @@
 
 #include "camel-test.h"
 
-#include <camel/camel-stream-fs.h>
-#include <camel/camel-stream-mem.h>
-#include <camel/camel-stream-filter.h>
-#include <camel/camel-mime-filter-crlf.h>
-
 #define d(x)
 
 #define NUM_CASES 1
@@ -30,7 +25,7 @@ main (gint argc, gchar **argv)
 {
 	CamelStream *source;
 	CamelStream *correct;
-	CamelStreamFilter *filter;
+	CamelStream *stream;
 	CamelMimeFilter *sh;
 	gchar *work;
 	gint i;
@@ -38,7 +33,7 @@ main (gint argc, gchar **argv)
 	gint comp_i;
 	gchar comp_correct[CHUNK_SIZE], comp_filter[CHUNK_SIZE];
 
-	camel_test_init(argc, argv);
+	camel_test_init (argc, argv);
 
 	for (i = 0; i < NUM_CASES; i++) {
 		gint j;
@@ -69,7 +64,7 @@ main (gint argc, gchar **argv)
 			}
 
 			camel_test_push ("Initializing objects");
-			source = camel_stream_fs_new_with_name (infile, 0, O_RDONLY);
+			source = camel_stream_fs_new_with_name (infile, 0, O_RDONLY, NULL);
 			if (!source) {
 				camel_test_fail ("Failed to open input case in \"%s\"", infile);
 				g_free (infile);
@@ -77,7 +72,7 @@ main (gint argc, gchar **argv)
 			}
 			g_free (infile);
 
-			correct = camel_stream_fs_new_with_name (outfile, 0, O_RDONLY);
+			correct = camel_stream_fs_new_with_name (outfile, 0, O_RDONLY, NULL);
 			if (!correct) {
 				camel_test_fail ("Failed to open correct output in \"%s\"", outfile);
 				g_free (outfile);
@@ -85,8 +80,8 @@ main (gint argc, gchar **argv)
 			}
 			g_free (outfile);
 
-			filter = camel_stream_filter_new_with_stream (CAMEL_STREAM (source));
-			if (!filter) {
+			stream = camel_stream_filter_new (source);
+			if (!stream) {
 				camel_test_fail ("Couldn't create CamelStreamFilter??");
 				continue;
 			}
@@ -97,7 +92,8 @@ main (gint argc, gchar **argv)
 				continue;
 			}
 
-			camel_stream_filter_add (filter, sh);
+			camel_stream_filter_add (
+				CAMEL_STREAM_FILTER (stream), sh);
 			camel_test_pull ();
 
 			camel_test_push ("Running filter and comparing to correct result");
@@ -105,7 +101,9 @@ main (gint argc, gchar **argv)
 			comp_progress = 0;
 
 			while (1) {
-				comp_correct_chunk = camel_stream_read (correct, comp_correct, CHUNK_SIZE);
+				comp_correct_chunk = camel_stream_read (
+					correct, comp_correct,
+					CHUNK_SIZE, NULL, NULL);
 				comp_filter_chunk = 0;
 
 				if (comp_correct_chunk == 0)
@@ -114,9 +112,11 @@ main (gint argc, gchar **argv)
 				while (comp_filter_chunk < comp_correct_chunk) {
 					gssize delta;
 
-					delta = camel_stream_read (CAMEL_STREAM (filter),
-								   comp_filter + comp_filter_chunk,
-								   CHUNK_SIZE - comp_filter_chunk);
+					delta = camel_stream_read (
+						stream,
+						comp_filter + comp_filter_chunk,
+						CHUNK_SIZE - comp_filter_chunk,
+						NULL, NULL);
 
 					if (delta == 0) {
 						camel_test_fail ("Chunks are different sizes: correct is %d, "
@@ -144,10 +144,10 @@ main (gint argc, gchar **argv)
 
 			/* inefficient */
 			camel_test_push ("Cleaning up");
-			camel_object_unref (CAMEL_OBJECT (filter));
-			camel_object_unref (CAMEL_OBJECT (correct));
-			camel_object_unref (CAMEL_OBJECT (source));
-			camel_object_unref (CAMEL_OBJECT (sh));
+			g_object_unref (stream);
+			g_object_unref (correct);
+			g_object_unref (source);
+			g_object_unref (sh);
 			camel_test_pull ();
 
 			camel_test_pull ();
