@@ -10,26 +10,23 @@
 
 #include "e-iterator.h"
 
-static void e_iterator_init (EIterator *card);
-static void e_iterator_class_init (EIteratorClass *klass);
-
 enum {
 	INVALIDATE,
 	LAST_SIGNAL
 };
 
-static guint e_iterator_signals [LAST_SIGNAL] = { 0, };
+static guint e_iterator_signals[LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (EIterator, e_iterator, G_TYPE_OBJECT);
+G_DEFINE_TYPE (EIterator, e_iterator, G_TYPE_OBJECT)
 
 static void
 e_iterator_class_init (EIteratorClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = G_OBJECT_CLASS(klass);
+	object_class = G_OBJECT_CLASS (klass);
 
-	e_iterator_signals [INVALIDATE] =
+	e_iterator_signals[INVALIDATE] =
 		g_signal_new ("invalidate",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -61,11 +58,11 @@ e_iterator_init (EIterator *card)
 /*
  * Virtual functions:
  */
-const void *
+gconstpointer
 e_iterator_get      (EIterator *iterator)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->get)
-		return E_ITERATOR_GET_CLASS(iterator)->get(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->get)
+		return E_ITERATOR_GET_CLASS (iterator)->get (iterator);
 	else
 		return NULL;
 }
@@ -73,22 +70,22 @@ e_iterator_get      (EIterator *iterator)
 void
 e_iterator_reset    (EIterator *iterator)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->reset)
-		E_ITERATOR_GET_CLASS(iterator)->reset(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->reset)
+		E_ITERATOR_GET_CLASS (iterator)->reset (iterator);
 }
 
 void
 e_iterator_last     (EIterator *iterator)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->last)
-		E_ITERATOR_GET_CLASS(iterator)->last(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->last)
+		E_ITERATOR_GET_CLASS (iterator)->last (iterator);
 }
 
 gboolean
 e_iterator_next     (EIterator *iterator)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->next)
-		return E_ITERATOR_GET_CLASS(iterator)->next(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->next)
+		return E_ITERATOR_GET_CLASS (iterator)->next (iterator);
 	else
 		return FALSE;
 }
@@ -96,8 +93,8 @@ e_iterator_next     (EIterator *iterator)
 gboolean
 e_iterator_prev     (EIterator *iterator)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->prev)
-		return E_ITERATOR_GET_CLASS(iterator)->prev(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->prev)
+		return E_ITERATOR_GET_CLASS (iterator)->prev (iterator);
 	else
 		return FALSE;
 }
@@ -105,24 +102,24 @@ e_iterator_prev     (EIterator *iterator)
 void
 e_iterator_delete   (EIterator *iterator)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->remove)
-		E_ITERATOR_GET_CLASS(iterator)->remove(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->remove)
+		E_ITERATOR_GET_CLASS (iterator)->remove (iterator);
 }
 
 void           e_iterator_insert     (EIterator  *iterator,
-				      const void *object,
+				      gconstpointer object,
 				      gboolean    before)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->insert)
-		E_ITERATOR_GET_CLASS(iterator)->insert(iterator, object, before);
+	if (E_ITERATOR_GET_CLASS (iterator)->insert)
+		E_ITERATOR_GET_CLASS (iterator)->insert (iterator, object, before);
 }
 
 void
 e_iterator_set      (EIterator *iterator,
 			  const void    *object)
 {
-	if (E_ITERATOR_GET_CLASS(iterator)->set)
-		E_ITERATOR_GET_CLASS(iterator)->set(iterator, object);
+	if (E_ITERATOR_GET_CLASS (iterator)->set)
+		E_ITERATOR_GET_CLASS (iterator)->set (iterator, object);
 }
 
 gboolean
@@ -131,8 +128,8 @@ e_iterator_is_valid (EIterator *iterator)
 	if (!iterator)
 		return FALSE;
 
-	if (E_ITERATOR_GET_CLASS(iterator)->is_valid)
-		return E_ITERATOR_GET_CLASS(iterator)->is_valid(iterator);
+	if (E_ITERATOR_GET_CLASS (iterator)->is_valid)
+		return E_ITERATOR_GET_CLASS (iterator)->is_valid (iterator);
 	else
 		return FALSE;
 }
@@ -143,5 +140,5 @@ e_iterator_invalidate (EIterator *iterator)
 	g_return_if_fail (iterator != NULL);
 	g_return_if_fail (E_IS_ITERATOR (iterator));
 
-	g_signal_emit (iterator, e_iterator_signals [INVALIDATE], 0);
+	g_signal_emit (iterator, e_iterator_signals[INVALIDATE], 0);
 }
