@@ -30,16 +30,15 @@
 
 #define ES_CLASS(obj)  E_SOURCE_CLASS (G_OBJECT_GET_CLASS (obj))
 
-
 /* Private members.  */
 
 struct _ESourcePrivate {
 	ESourceGroup *group;
 
-	char *uid;
-	char *name;
-	char *relative_uri;
-	char *absolute_uri;
+	gchar *uid;
+	gchar *name;
+	gchar *relative_uri;
+	gchar *absolute_uri;
 
 	gboolean readonly;
 
@@ -48,15 +47,13 @@ struct _ESourcePrivate {
 	GHashTable *properties;
 };
 
-
 /* Signals.  */
 
 enum {
 	CHANGED,
 	LAST_SIGNAL
 };
-static unsigned int signals[LAST_SIGNAL] = { 0 };
-
+static guint signals[LAST_SIGNAL] = { 0 };
 
 /* Callbacks.  */
 
@@ -69,7 +66,6 @@ group_weak_notify (ESource *source,
 	g_signal_emit (source, signals[CHANGED], 0);
 }
 
-
 /* GObject methods.  */
 
 G_DEFINE_TYPE (ESource, e_source, G_TYPE_OBJECT)
@@ -105,7 +101,6 @@ impl_dispose (GObject *object)
 	(* G_OBJECT_CLASS (e_source_parent_class)->dispose) (object);
 }
 
-
 /* Initialization.  */
 
 static void
@@ -163,8 +158,8 @@ set_color_spec (ESource *source,
 /* Public methods.  */
 
 ESource *
-e_source_new  (const char *name,
-	       const char *relative_uri)
+e_source_new  (const gchar *name,
+	       const gchar *relative_uri)
 {
 	ESource *source;
 
@@ -180,8 +175,8 @@ e_source_new  (const char *name,
 }
 
 ESource *
-e_source_new_with_absolute_uri (const char *name,
-				const char *absolute_uri)
+e_source_new_with_absolute_uri (const gchar *name,
+				const gchar *absolute_uri)
 {
 	ESource *source;
 
@@ -208,7 +203,7 @@ e_source_new_from_xml_node (xmlNodePtr node)
 
 	source = g_object_new (e_source_get_type (), NULL);
 
-	source->priv->uid = g_strdup ((char*)uid);
+	source->priv->uid = g_strdup ((gchar *)uid);
 	xmlFree (uid);
 
 	if (e_source_update_from_xml_node (source, node, NULL))
@@ -228,14 +223,14 @@ import_properties (ESource *source,
 	for (prop_node = prop_root->children; prop_node; prop_node = prop_node->next) {
 		xmlChar *name, *value;
 
-		if (!prop_node->name || strcmp ((char*)prop_node->name, "property"))
+		if (!prop_node->name || strcmp ((gchar *)prop_node->name, "property"))
 			continue;
 
 		name = xmlGetProp (prop_node, (xmlChar*)"name");
 		value = xmlGetProp (prop_node, (xmlChar*)"value");
 
 		if (name && value)
-			g_hash_table_insert (priv->properties, g_strdup ((char*)name), g_strdup ((char*)value));
+			g_hash_table_insert (priv->properties, g_strdup ((gchar *)name), g_strdup ((gchar *)value));
 
 		if (name)
 			xmlFree (name);
@@ -279,7 +274,7 @@ compare_str_hashes (GHashTable *table1, GHashTable *table2)
  *
  * Update the ESource properties from @node.
  *
- * Return value: %TRUE if the data in @node was recognized and parsed into
+ * Returns: %TRUE if the data in @node was recognized and parsed into
  * acceptable values for @source, %FALSE otherwise.
  **/
 gboolean
@@ -308,28 +303,52 @@ e_source_update_from_xml_node (ESource *source,
 		goto done;
 
 	if (source->priv->name == NULL
-	    || strcmp ((char*)name, source->priv->name) != 0
-	    || source->priv->relative_uri == NULL
-	    || relative_uri != NULL
-	    || strcmp ((char*)relative_uri, source->priv->relative_uri) != 0) {
+	    || strcmp ((gchar *)name, source->priv->name) != 0
+	    || (source->priv->relative_uri == NULL && relative_uri != NULL)
+	    || (source->priv->relative_uri != NULL && relative_uri == NULL)
+	    || (relative_uri && source->priv->relative_uri && strcmp ((gchar *)relative_uri, source->priv->relative_uri) != 0)) {
+		gchar *abs_uri = NULL;
+
 		g_free (source->priv->name);
-		source->priv->name = g_strdup ((char*)name);
+		source->priv->name = g_strdup ((gchar *)name);
+
+		if (source->priv->group) {
+			abs_uri = e_source_build_absolute_uri (source);
+		}
+
+		if (abs_uri && source->priv->absolute_uri && g_str_equal (abs_uri, source->priv->absolute_uri)) {
+			/* reset the absolute uri to NULL to be regenerated when asked for,
+			   but only when it was generated also before */
+			g_free (source->priv->absolute_uri);
+			source->priv->absolute_uri = NULL;
+		} else if (source->priv->absolute_uri &&
+			   source->priv->relative_uri &&
+			   g_str_has_suffix (source->priv->absolute_uri, source->priv->relative_uri)) {
+			gchar *tmp = source->priv->absolute_uri;
+
+			tmp[strlen (tmp) - strlen (source->priv->relative_uri)] = 0;
+			source->priv->absolute_uri = g_strconcat (tmp, (gchar *)relative_uri, NULL);
+
+			g_free (tmp);
+		}
+
+		g_free (abs_uri);
 
 		g_free (source->priv->relative_uri);
-		source->priv->relative_uri = g_strdup ((char*)relative_uri);
+		source->priv->relative_uri = g_strdup ((gchar *)relative_uri);
 
 		changed = TRUE;
 	}
 
 	if (absolute_uri != NULL) {
 		g_free (source->priv->absolute_uri);
-		source->priv->absolute_uri = g_strdup ((char*)absolute_uri);
+		source->priv->absolute_uri = g_strdup ((gchar *)absolute_uri);
 		changed = TRUE;
 	}
 
 	if (color == NULL) {
 		/* It is okay for color_spec to be NULL. */
-		changed |= set_color_spec (source, (char*)color_spec);
+		changed |= set_color_spec (source, (gchar *)color_spec);
 	} else {
 		gchar buffer[8];
 		g_snprintf (buffer, sizeof (buffer), "#%s", color);
@@ -347,7 +366,7 @@ e_source_update_from_xml_node (ESource *source,
 		if (!node->name)
 			continue;
 
-		if (!strcmp ((char*)node->name, "properties")) {
+		if (!strcmp ((gchar *)node->name, "properties")) {
 			GHashTable *temp = source->priv->properties;
 			source->priv->properties = g_hash_table_new_full (g_str_hash, g_str_equal,
 									  g_free, g_free);
@@ -389,24 +408,24 @@ e_source_update_from_xml_node (ESource *source,
  * Assuming that @node is a valid ESource specification, retrieve the name of
  * the source from it.
  *
- * Return value: Name of the source in the specified @node.  The caller must
+ * Returns: Name of the source in the specified @node.  The caller must
  * free the string.
  **/
-char *
+gchar *
 e_source_uid_from_xml_node (xmlNodePtr node)
 {
 	xmlChar *uid = xmlGetProp (node, (xmlChar*)"uid");
-	char *retval;
+	gchar *retval;
 
 	if (uid == NULL)
 		return NULL;
 
-	retval = g_strdup ((char*)uid);
+	retval = g_strdup ((gchar *)uid);
 	xmlFree (uid);
 	return retval;
 }
 
-char *
+gchar *
 e_source_build_absolute_uri (ESource *source)
 {
 	const gchar *base_uri_str;
@@ -461,7 +480,7 @@ e_source_set_group (ESource *source,
 
 void
 e_source_set_name (ESource *source,
-		   const char *name)
+		   const gchar *name)
 {
 	g_return_if_fail (E_IS_SOURCE (source));
 	g_return_if_fail (name != NULL);
@@ -481,38 +500,45 @@ e_source_set_name (ESource *source,
 
 void
 e_source_set_relative_uri (ESource *source,
-			   const char *relative_uri)
+			   const gchar *relative_uri)
 {
-	char *absolute_uri;
+	gchar *absolute_uri, *old_abs_uri = NULL;
 
 	g_return_if_fail (E_IS_SOURCE (source));
 
 	if (source->priv->readonly)
 		return;
 
-	if (source->priv->relative_uri == relative_uri)
+	if (source->priv->relative_uri == relative_uri ||
+	    (source->priv->relative_uri && relative_uri && g_str_equal (source->priv->relative_uri, relative_uri)))
 		return;
 
+	if (source->priv->group)
+		old_abs_uri = e_source_build_absolute_uri (source);
+
 	g_free (source->priv->relative_uri);
 	source->priv->relative_uri = g_strdup (relative_uri);
 
-	/* reset the absolute uri */
+	/* reset the absolute uri, if it's a generated one */
 	if (source->priv->absolute_uri &&
+	    (!old_abs_uri || g_str_equal (source->priv->absolute_uri, old_abs_uri)) &&
 	    (absolute_uri = e_source_build_absolute_uri (source))) {
 		g_free (source->priv->absolute_uri);
 		source->priv->absolute_uri = absolute_uri;
 	}
 
+	g_free (old_abs_uri);
+
 	g_signal_emit (source, signals[CHANGED], 0);
 }
 
 void
 e_source_set_absolute_uri (ESource *source,
-			   const char *absolute_uri)
+			   const gchar *absolute_uri)
 {
 	g_return_if_fail (E_IS_SOURCE (source));
 
-	if (!!absolute_uri == !!source->priv->absolute_uri
+	if ((absolute_uri == source->priv->absolute_uri && absolute_uri == NULL)
 	    || (absolute_uri && source->priv->absolute_uri && !strcmp (source->priv->absolute_uri, absolute_uri)))
 		return;
 
@@ -537,28 +563,6 @@ e_source_set_readonly (ESource  *source,
 
 }
 
-#ifndef EDS_DISABLE_DEPRECATED
-void
-e_source_set_color (ESource *source,
-		    guint32 color)
-{
-	gchar color_spec[8];
-
-	g_return_if_fail (E_IS_SOURCE (source));
-
-	g_snprintf (color_spec, sizeof (color_spec), "#%06x", color);
-	e_source_set_color_spec (source, color_spec);
-}
-
-void
-e_source_unset_color (ESource *source)
-{
-	g_return_if_fail (E_IS_SOURCE (source));
-
-	e_source_set_color_spec (source, NULL);
-}
-#endif
-
 /**
  * e_source_set_color_spec:
  * @source: an ESource
@@ -572,7 +576,7 @@ e_source_unset_color (ESource *source)
  **/
 void
 e_source_set_color_spec (ESource *source,
-		         const gchar *color_spec)
+			 const gchar *color_spec)
 {
 	g_return_if_fail (E_IS_SOURCE (source));
 
@@ -588,7 +592,7 @@ e_source_peek_group (ESource *source)
 	return source->priv->group;
 }
 
-const char *
+const gchar *
 e_source_peek_uid (ESource *source)
 {
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -596,7 +600,7 @@ e_source_peek_uid (ESource *source)
 	return source->priv->uid;
 }
 
-const char *
+const gchar *
 e_source_peek_name (ESource *source)
 {
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -604,7 +608,7 @@ e_source_peek_name (ESource *source)
 	return source->priv->name;
 }
 
-const char *
+const gchar *
 e_source_peek_relative_uri (ESource *source)
 {
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -612,7 +616,7 @@ e_source_peek_relative_uri (ESource *source)
 	return source->priv->relative_uri;
 }
 
-const char *
+const gchar *
 e_source_peek_absolute_uri (ESource *source)
 {
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -627,11 +631,11 @@ e_source_peek_absolute_uri (ESource *source)
  * Return the textual representation of the color for @source, or %NULL if it
  * has none.  The returned string should be parsable by #gdk_color_parse().
  *
- * Return value: a string specifying the color
+ * Returns: a string specifying the color
  *
  * Since: 1.10
  **/
-const char *
+const gchar *
 e_source_peek_color_spec (ESource *source)
 {
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -647,44 +651,7 @@ e_source_get_readonly (ESource *source)
 	return source->priv->readonly;
 }
 
-
-#ifndef EDS_DISABLE_DEPRECATED
-/**
- * e_source_get_color:
- * @source: An ESource
- * @color_return: Pointer to a variable where the returned color will be
- * stored.
- *
- * If @source has an associated color, return it in *@color_return.
- *
- * Return value: %TRUE if the @source has a defined color (and hence
- * *@color_return was set), %FALSE otherwise.
- **/
-gboolean
-e_source_get_color (ESource *source,
-		    guint32 *color_return)
-{
-	const gchar *color_spec;
-	guint32 color;
-
-	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
-
-	color_spec = e_source_peek_color_spec (source);
-
-	if (color_spec == NULL)
-		return FALSE;
-
-	if (sscanf (color_spec, "#%06x", &color) != 1)
-		return FALSE;
-
-	if (color_return != NULL)
-		*color_return = color;
-
-	return TRUE;
-}
-#endif
-
-char *
+gchar *
 e_source_get_uri (ESource *source)
 {
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -702,7 +669,6 @@ e_source_get_uri (ESource *source)
 		return e_source_build_absolute_uri (source);
 }
 
-
 static void
 property_dump_cb (const xmlChar *key, const xmlChar *value, xmlNodePtr root)
 {
@@ -713,14 +679,13 @@ property_dump_cb (const xmlChar *key, const xmlChar *value, xmlNodePtr root)
 	xmlSetProp (node, (xmlChar*)"value", value);
 }
 
-
 static xmlNodePtr
 dump_common_to_xml_node (ESource *source,
 			 xmlNodePtr parent_node)
 {
 	ESourcePrivate *priv;
 	xmlNodePtr node;
-	const char *abs_uri = NULL, *relative_uri = NULL;
+	const gchar *abs_uri = NULL, *relative_uri = NULL;
 
 	priv = source->priv;
 
@@ -751,7 +716,6 @@ dump_common_to_xml_node (ESource *source,
 	return node;
 }
 
-
 void
 e_source_dump_to_xml_node (ESource *source,
 			   xmlNodePtr parent_node)
@@ -761,15 +725,14 @@ e_source_dump_to_xml_node (ESource *source,
 	dump_common_to_xml_node (source, parent_node);
 }
 
-
-char *
+gchar *
 e_source_to_standalone_xml (ESource *source)
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
 	xmlChar *xml_buffer;
-	char *returned_buffer;
-	int xml_buffer_size;
+	gchar *returned_buffer;
+	gint xml_buffer_size;
 	gchar *uri;
 
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
@@ -788,7 +751,7 @@ e_source_to_standalone_xml (ESource *source)
 
 	returned_buffer = g_malloc (xml_buffer_size + 1);
 	memcpy (returned_buffer, xml_buffer, xml_buffer_size);
-	returned_buffer [xml_buffer_size] = '\0';
+	returned_buffer[xml_buffer_size] = '\0';
 	xmlFree (xml_buffer);
 
 	return returned_buffer;
@@ -801,53 +764,55 @@ e_source_to_standalone_xml (ESource *source)
  *
  * Compares if @a is equivalent to @b.
  *
- * Return value: %TRUE if @a is equivalent to @b, 
+ * Returns: %TRUE if @a is equivalent to @b,
  * %FALSE otherwise.
+ *
+ * Since: 2.24
  **/
-gboolean 
+gboolean
 e_source_equal (ESource *a, ESource *b)
 {
-	g_return_val_if_fail (E_IS_SOURCE (a), FALSE); 
-	g_return_val_if_fail (E_IS_SOURCE (b), FALSE); 
+	g_return_val_if_fail (E_IS_SOURCE (a), FALSE);
+	g_return_val_if_fail (E_IS_SOURCE (b), FALSE);
 
 	#define ONLY_ONE_NULL(aa, bb) (((aa) == NULL && (bb) != NULL) || ((aa) != NULL && (bb) == NULL))
 
 	/* Compare source stuff */
-	if (a->priv->uid 
-	 && b->priv->uid 
+	if (a->priv->uid
+	 && b->priv->uid
 	 && g_ascii_strcasecmp (a->priv->uid, b->priv->uid))
-		return FALSE; 
+		return FALSE;
 
-	if (a->priv->name 
-	 && b->priv->name 
+	if (a->priv->name
+	 && b->priv->name
 	 && g_ascii_strcasecmp (a->priv->name, b->priv->name))
-		return FALSE; 
+		return FALSE;
 
-	if (a->priv->relative_uri 
-	 && b->priv->relative_uri 
+	if (a->priv->relative_uri
+	 && b->priv->relative_uri
 	 && g_ascii_strcasecmp (a->priv->relative_uri, b->priv->relative_uri))
-		return FALSE; 
+		return FALSE;
 
-	if (a->priv->absolute_uri 
-	 && b->priv->absolute_uri 
+	if (a->priv->absolute_uri
+	 && b->priv->absolute_uri
 	 && g_ascii_strcasecmp (a->priv->absolute_uri, b->priv->absolute_uri))
-		return FALSE; 
+		return FALSE;
 
-	if ((a->priv->color_spec 
-	 && b->priv->color_spec 
+	if ((a->priv->color_spec
+	 && b->priv->color_spec
 	 && g_ascii_strcasecmp (a->priv->color_spec, b->priv->color_spec)) ||
 	 (ONLY_ONE_NULL (a->priv->color_spec, b->priv->color_spec)))
-		return FALSE; 
+		return FALSE;
 
 	if (a->priv->readonly != b->priv->readonly)
-		return FALSE; 
+		return FALSE;
 
 	if (!compare_str_hashes (a->priv->properties, b->priv->properties))
-		return FALSE; 
+		return FALSE;
 
 	#undef ONLY_ONE_NULL
 
-	return TRUE; 
+	return TRUE;
 }
 
 /**
@@ -857,28 +822,30 @@ e_source_equal (ESource *a, ESource *b)
  *
  * Compares if @a is equivalent to @b.
  *
- * Return value: %TRUE if @a is equivalent to @b, 
+ * Returns: %TRUE if @a is equivalent to @b,
  * %FALSE otherwise.
+ *
+ * Since: 2.24
  **/
-gboolean 
+gboolean
 e_source_xmlstr_equal (const gchar *a, const gchar *b)
 {
-	ESource *srca, *srcb; 
-	gboolean retval; 
+	ESource *srca, *srcb;
+	gboolean retval;
 
-	srca = e_source_new_from_standalone_xml (a); 
-	srcb = e_source_new_from_standalone_xml (b); 
+	srca = e_source_new_from_standalone_xml (a);
+	srcb = e_source_new_from_standalone_xml (b);
 
-	retval = e_source_equal (srca, srcb); 
+	retval = e_source_equal (srca, srcb);
 
-	g_object_unref (srca); 
-	g_object_unref (srcb); 
+	g_object_unref (srca);
+	g_object_unref (srcb);
 
-	return retval; 
+	return retval;
 }
 
 ESource *
-e_source_new_from_standalone_xml (const char *xml)
+e_source_new_from_standalone_xml (const gchar *xml)
 {
 	xmlDocPtr doc;
 	xmlNodePtr root;
@@ -889,7 +856,7 @@ e_source_new_from_standalone_xml (const char *xml)
 		return NULL;
 
 	root = doc->children;
-	if (strcmp ((char*)root->name, "source") != 0)
+	if (strcmp ((gchar *)root->name, "source") != 0)
 		return NULL;
 
 	source = e_source_new_from_xml_node (root);
@@ -898,7 +865,6 @@ e_source_new_from_standalone_xml (const char *xml)
 	return source;
 }
 
-
 const gchar *
 e_source_get_property (ESource *source,
 		       const gchar *property)
@@ -911,15 +877,20 @@ e_source_get_property (ESource *source,
 	return g_hash_table_lookup (priv->properties, property);
 }
 
-char *
-e_source_get_duped_property (ESource *source, const char *property)
+/**
+ * e_source_get_duped_property:
+ *
+ * Since: 1.12
+ **/
+gchar *
+e_source_get_duped_property (ESource *source, const gchar *property)
 {
 	ESourcePrivate *priv;
 
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
 	priv = source->priv;
 
- 	return g_strdup (g_hash_table_lookup (priv->properties, property));
+	return g_strdup (g_hash_table_lookup (priv->properties, property));
 }
 
 void
@@ -940,7 +911,6 @@ e_source_set_property (ESource *source,
 	g_signal_emit (source, signals[CHANGED], 0);
 }
 
-
 void
 e_source_foreach_property (ESource *source, GHFunc func, gpointer data)
 {
@@ -952,14 +922,12 @@ e_source_foreach_property (ESource *source, GHFunc func, gpointer data)
 	g_hash_table_foreach (priv->properties, func, data);
 }
 
-
 static void
 copy_property (const gchar *key, const gchar *value, ESource *new_source)
 {
 	e_source_set_property (new_source, key, value);
 }
 
-
 ESource *
 e_source_copy (ESource *source)
 {
